#! /usr/bin/python

from PyQt4 import QtCore, QtGui

class InstallerTreeItem(QtCore.QObject):
    # Signals
    checkStateChangeRequested = QtCore.pyqtSignal(QtCore.QObject, QtCore.QAbstractItemModel)
    checkStateChanged = QtCore.pyqtSignal(QtCore.QObject)
    
    def __init__(self, data, parent=None, root=None):
        super(self.__class__, self).__init__(parent)

        self.rootItem = root
        if root == None:
            self.rootItem = self
            
        self.parentItem = parent
        self.itemData = data
        self.childItems = []
        self.radioGroup = []
        self.checkType = None
        self.checkState = False
    
    def parseData(self):
        checkboxtype = self.getData("CheckBoxType", "").lower()
        if checkboxtype == "checkbox":
            self.checkType = 1
        elif checkboxtype == "radio":
            self.checkType = 2
            if self.parentItem:
                self.parentItem.radioGroup.append(self)

        if self.checkType not in [None, ""]:
            # Prevent bad configs causing multiple radiobuttons to be checked.
            self.setChecked((self.getData("Checked", False)), True)

    def hasData(self, name):
        return name in self.itemData
    
    def getData(self, name, default=None):
        if self.hasData(name):
            return self.itemData[name]
        
        return default
    
    def isChecked(self):
        return self.checkState
    
    def setChecked(self, state, force=False):
        if self.checkState == state:
            # No change needed
            return
        
        # The 'force' param is only used internal to TreeItem, here. All other
        # code must ignore it.
        if self.checkType is 2:
            # Enforce 'exclusivity' in radiogroups
            if state == True:
                # Uncheck any other items in the parent's radiogroup
                self.parentItem.clearRadioSelections(self)
            elif not force:
                return

        self.checkState = state
        self.checkStateChanged.emit(self)

    def clearRadioSelections(self, ignore=None):
        for item in self.radioGroup:
            if item is ignore:
                continue
            # 'Exclusive' implementation for radiobuttons
            item.setChecked(False, True)
            
    def appendChild(self, item):
        self.childItems.append(item)

    def child(self, row):
        return self.childItems[row]

    def childCount(self):
        return len(self.childItems)

    def columnCount(self):
        return 2

    def data(self, column):
        if column == 0:
            return self.getData("Title")
        elif column == 1:
            return self.getData("Summary")

    def parent(self):
        return self.parentItem

    def row(self):
        if self.parentItem and self in self.parentItem.childItems:
            return self.parentItem.childItems.index(self)

        return 0

class InstallerTreeModel(QtCore.QAbstractItemModel):
    CBTYPE_NONE = None
    CBTYPE_CHECKBOX = 1
    CBTYPE_RADIO = 2
    
    newItemAdded = QtCore.pyqtSignal(InstallerTreeItem, InstallerTreeItem) # Parent, New Item
    
    def __init__(self, parent=None):
        super(self.__class__, self).__init__(parent)

    def dataInit(self, data):
        self.rootItem = InstallerTreeItem(data)
        return self.parseData(data, self.rootItem)

    def parseData(self, data, curitem):
        curitem.parseData()
        if "SubItems" in data:
            for item in data["SubItems"]:
                newitem = InstallerTreeItem(item, curitem, curitem.rootItem)
                self.newItemAdded.emit(curitem, newitem)
                curitem.appendChild(newitem)
                self.parseData(item, newitem)
        
        return curitem

    def columnCount(self, parent):
        return 2
        
    def data(self, index, role):
        if not index.isValid():
            return None

        item = index.internalPointer()

        if role == QtCore.Qt.DisplayRole:
            return item.data(index.column())
        elif role == QtCore.Qt.CheckStateRole and \
             item.checkType is not None and \
             index.column() == 0:
            return QtCore.Qt.Checked if item.isChecked() else QtCore.Qt.Unchecked

        return None
            
    def setData(self, index, value, role):
        status = False
        
        if role == QtCore.Qt.CheckStateRole:
            item = index.internalPointer()
            item.checkStateChangeRequested.emit(item, self)
            status = True
            self.refreshList()
                
        return status

    def flags(self, index):
        if not index.isValid():
            return QtCore.Qt.NoItemFlags
        
        flags = QtCore.Qt.ItemIsEnabled
        item = index.internalPointer()
        
        if item.checkType is not None:
            flags |= QtCore.Qt.ItemIsUserCheckable
        
        return flags

    def headerData(self, section, orientation, role):
        if orientation == QtCore.Qt.Horizontal and role == QtCore.Qt.DisplayRole:
            return self.rootItem.data(section)

        return None

    def refreshList(self):
        self.layoutChanged.emit()
    
    def index(self, row, column, parent):
        if not self.hasIndex(row, column, parent):
            return QtCore.QModelIndex()

        if not parent.isValid():
            parentItem = self.rootItem
        else:
            parentItem = parent.internalPointer()

        childItem = parentItem.child(row)
        
        if childItem:
            return self.createIndex(row, column, childItem)
        else:
            return QtCore.QModelIndex()

    def parent(self, index):
        if not index.isValid():
            return QtCore.QModelIndex()

        childItem = index.internalPointer()
        parentItem = childItem.parent()

        if parentItem == self.rootItem:
            return QtCore.QModelIndex()

        return self.createIndex(parentItem.row(), 0, parentItem)

    def rowCount(self, parent):
        if parent.column() > 0:
            return 0

        if not parent.isValid():
            parentItem = self.rootItem
        else:
            parentItem = parent.internalPointer()

        return parentItem.childCount()

class InstallItemDelegate(QtGui.QItemDelegate):
    def paint(self, painter, option, index):
        # Save the current item for later, to allow us to differentiate between
        # what kind of checkbox to draw. Purely aesthetic.
        self.currentItem = index.internalPointer()
        super(self.__class__, self).paint(painter, option, index)

    def drawCheck( self, painter, option, rect, state ):
        if not rect.isValid(): 
            return

        option.rect = rect
        option.state &= ~QtGui.QStyle.State_HasFocus
        option.state |= {
            QtCore.Qt.Unchecked: QtGui.QStyle.State_Off,
            QtCore.Qt.PartiallyChecked: QtGui.QStyle.State_NoChange,
            QtCore.Qt.Checked: QtGui.QStyle.State_On
        }[ state ]

        if self.view:
            style = self.view.style()
        else:
            style = QtGui.QApplication.style()
        
        if self.currentItem.checkType == 2:
            style.drawPrimitive( QtGui.QStyle.PE_IndicatorRadioButton, option, painter, self.view )
        else:
            style.drawPrimitive( QtGui.QStyle.PE_IndicatorViewItemCheck, option, painter, self.view )
        
    